#
# spec file for package dwarves
#
# Copyright (c) 2024 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%if (0%{?sle_version} && 0%{?sle_version} >= 150600) || (0%{?suse_version} && 0%{?suse_version} > 1500)
# have libbpf >= 1.0.1 and linux-glibc-devel >= 5.16
%define have_libbpf 1
%endif
%if (0%{?sle_version} && 0%{?sle_version} <= 150300) || (0%{?suse_version} && 0%{?suse_version} < 1500)
%define have_libebl_plugins 1
%endif

Name:           dwarves
Version:        1.26
Release:        1.2
Summary:        DWARF utilities
License:        GPL-2.0-only
Group:          Development/Tools/Debuggers
URL:            https://acmel.wordpress.com/
#Git-Clone:	git://git.kernel.org/pub/scm/devel/pahole/pahole
#Git-Web:	http://git.kernel.org/cgit/devel/pahole/pahole.git
Source:         https://fedorapeople.org/~acme/dwarves/dwarves-%version.tar.xz
Source2:        https://fedorapeople.org/~acme/dwarves/dwarves-%version.tar.sign
Source8:        dwarves.keyring
Source9:        baselibs.conf
BuildRequires:  cmake
BuildRequires:  libdw-devel >= 0.171
BuildRequires:  libelf-devel
BuildRequires:  pkgconfig
%if 0%{?have_libbpf}
BuildRequires:  linux-glibc-devel >= 5.16
BuildRequires:  pkgconfig(libbpf) >= 1.0.0
%endif
BuildRequires:  pkgconfig(zlib)
# Also known by its most prominent tool
Provides:       pahole = %version-%release
%if 0%{?have_libebl_plugins}
BuildRequires:  libebl-devel
%endif

%description
dwarves is a set of tools that use the DWARF debugging information
inserted in ELF binaries by compilers such as GCC, used by well known
debuggers such as GDB, and more recent ones such as systemtap.

Utilities in the dwarves suite include pahole, that can be used to
find alignment holes in structs and classes in languages such as C,
C++, but not limited to these.

It also extracts other information such as CPU cacheline alignment,
helping pack those structures to achieve more cache hits.

A diff like tool, codiff can be used to compare the effects changes
in source code generate on the resulting binaries.

Another tool is pfunct, that can be used to find all sorts of
information about functions, inlines, decisions made by the compiler
about inlining, etc.

%package -n libdwarves1
Summary:        DWARF processing libraries of dwarves tools
Group:          System/Libraries
%if 0%{?have_libebl_plugins}
Requires:       libebl-plugins
%endif

%description -n libdwarves1
This package contains the libdwarves shared library for the dwarves
toolset, providing processing for DWARF, a debugging data format
for ELF files.

dwarves is a set of tools that use the DWARF debugging information
inserted in ELF binaries by compilers such as GCC, used by well known
debuggers such as GDB, and more recent ones such as systemtap.

%package -n libdwarves-devel
Summary:        DWARF processing library development files
Group:          Development/Libraries/C and C++
Requires:       libdwarves1 = %version-%release

%description -n libdwarves-devel
This package contains the development files for libdwarves, a library
for processing DWARF, a debugging data format for ELF files.

%prep
%autosetup -p1

%build
sv="$PWD/lib.v"
ver=$(echo %version | cut -d+ -f1)
echo "DWARVES_$ver{ global: *; };" >"$sv"
%cmake \
%if 0%{?have_libbpf}
	-DLIBBPF_EMBEDDED=OFF \
%endif
	-DCMAKE_SHARED_LINKER_FLAGS:STRING="-Wl,--version-script=$sv"
%cmake_build

%install
%cmake_install

%post   -n libdwarves1 -p /sbin/ldconfig
%postun -n libdwarves1 -p /sbin/ldconfig

%files
%doc README NEWS
%_bindir/*
%_mandir/man*/*

%files -n libdwarves1
%_libdir/*.so.1*

%files -n libdwarves-devel
%_libdir/*.so
%_includedir/*
%_datadir/%name

%changelog
* Wed Feb 28 2024 Jan Engelhardt <jengelh@inai.de>
- Update to release 1.26
  * Show more holes (the ones in contained types)
  * Ability to express the BTF features to encode
* Mon Oct  9 2023 Michal Suchanek <msuchanek@suse.com>
- Fix have_libbpf conditional (jsc#PED-4593 bsc#1216042).
* Sat Apr  8 2023 Jan Engelhardt <jengelh@inai.de>
- Update to release 1.25
  * Fixed the handling of DW_TAG_unspecified type in assembly
    functions that came with binutils 2.40
  * Allow the BTF encoding of optimized functions (ending in
    .constprop, .isra),
  * Exclude BTF optimized functions not following the calling
    convention
  * Support for DW_TAG_atomic_type
  * Support for the DW_TAG_LLVM_annotation BTF equivalent
    (BTF_KIND_TYPE_TAG) on the BTF loader and for now suppressing
    it when pretty printing
- Drop 0001-pahole-Support-lang-lang_exclude-asm.patch,
  0002-btf_encoder-Add-extra-debug-info-for-unsupported-DWA.patch,
  0003-btf_encoder-Store-the-CU-being-processed-to-avoid-ch.patch,
  0004-core-Add-DW_TAG_unspecified_type-to-tag__is_tag_type.patch,
  0005-core-Record-if-a-CU-has-a-DW_TAG_unspecified_type.patch,
  0006-btf_encoder-Encode-DW_TAG_unspecified_type-returning.patch,
  0007-dwarves-Zero-initialize-struct-cu-in-cu__new-to-prev.patch
* Fri Feb 17 2023 Martin Liška <mliska@suse.cz>
- Add 0001-pahole-Support-lang-lang_exclude-asm.patch,
  0002-btf_encoder-Add-extra-debug-info-for-unsupported-DWA.patch,
  0003-btf_encoder-Store-the-CU-being-processed-to-avoid-ch.patch,
  0004-core-Add-DW_TAG_unspecified_type-to-tag__is_tag_type.patch,
  0005-core-Record-if-a-CU-has-a-DW_TAG_unspecified_type.patch,
  0006-btf_encoder-Encode-DW_TAG_unspecified_type-returning.patch,
  0007-dwarves-Zero-initialize-struct-cu-in-cu__new-to-prev.patch
  as in order to support DW_TAG_unspecified_type that is newly
  emitted by binutils 2.40+.
* Tue Oct  4 2022 Dirk Müller <dmueller@suse.com>
- update to 1.24:
  * Add support to BTF_KIND_ENUM64 to represent enumeration
    entries with more than 32 bits.
  * Support multithreaded encoding, in addition to DWARF
    multithreaded loading, speeding up the process. Selected just
    like DWARF multithreaded loading, using the 'pahole -j'
    option.
  * Encode 'char' type as signed.
  * Add support to BTF_KIND_ENUM64.
  * Introduce --lang and --lang_exclude to specify the language
    the DWARF compile units were originated from to use or
    filter.
  * Introduce --compile to generate compilable code in a similar
    fashion to: bpftool btf dump file vmlinux format c >
    vmlinux.h As with 'bpftool', this will notice type shadowing,
    i.e. multiple types with the same name and will disambiguate
    by adding a suffix.
  * Don't segfault when processing bogus files.
- add dwarves.keyring for source validation
* Wed Dec  8 2021 Jan Engelhardt <jengelh@inai.de>
- Update to release 1.23
  * Ability to encode BTF tags.
  * Ability to carry attributes to the kernel BPF verifier for
    further checks.
  * Inference of struct member unnatural alignment
    (__attribute__(__aligned__(N))) to help in generating
    compileable headers matching the original type layout from
    BTF data.
* Mon Aug 23 2021 Jan Engelhardt <jengelh@inai.de>
- Update to release 1.22 (bsc#1190340).
  * Ability to encode BTF to a separate file
  * Multithreaded DWARF loading
  * Details at https://marc.info/?l=dwarves&m=162975403628056&w=2
* Wed Aug 18 2021 Michal Suchanek <msuchanek@suse.com>
- When building with system libbpf depend also on new enough Linux headers.
* Fri Aug 13 2021 msuchanek@suse.com
- 15.4 has new elfutils, drop libebl-plugins dependency.
- Update to version 1.21+git177.1ef87b2:
  * Fix crash with split BTF debuginfo
  * CMakeList.txt: Bump version to the upcoming 1.22 release, not out of the door yet
  - Remove upstreamed patch dwarves-Prep-1.22.patch
* Sat Jul 17 2021 Michal Suchanek <msuchanek@suse.com>
- Use system libbpf when available (bsc#1188420 bsc#1188419 jsc#SLE-17288 jsc#SLE-18805).
* Thu Jul 15 2021 msuchanek@suse.com
- Update to version 1.21+git175.1ef87b2 (jsc#SLE-17288):
  * libbpf: bump dependency to >= 0.4.0
  * CMakeLists.txt: Enable SHARED and STATIC lib creation
  * libbpf: Allow to use packaged version
  * btfdiff: Support diffing DWARF vs detached BTF
  * pahole: Allow encoding BTF into a detached file
  * pahole: Add --kabi_prefix flag
  * btf_encoder: fix and complete filtering out zero-sized per-CPU variables
  * btf: Remove ftrace filter
  * pahole: Introduce --with_flexible_array option to show just types ending in a flexible array
  * Many updates to btf_encoder and bug fixes
- Remove upstreamed patches
  - btf_encoder-fix-and-complete-filtering-out-zero-sized-per-CPU-variables.patch
  - remove-ftrace-filter.patch
- Pretend version 1.22 to make kernel happy
  + dwarves-Prep-1.22.patch
* Wed Jun  2 2021 Dirk Müller <dmueller@suse.com>
- use https for url (and more beautification changes from spec-cleaner)
* Tue Jun  1 2021 Michal Suchanek <msuchanek@suse.com>
- Handle zero sized per-cpu veriables in Linux BTF.
  + btf_encoder-fix-and-complete-filtering-out-zero-sized-per-CPU-variables.patch
* Mon May  3 2021 Jiri Slaby <jslaby@suse.cz>
- add remove-ftrace-filter.patch
* Thu Apr 22 2021 Jan Engelhardt <jengelh@inai.de>
- Codify build requirement on elfutils >= 0.171
* Mon Apr 12 2021 Jan Engelhardt <jengelh@inai.de>
- Update to release 1.21
  * Handle DWARF5 DW_OP_addrx and
    subprogram ret type with abstract_origin properly.
* Fri Feb  5 2021 Jan Engelhardt <jengelh@inai.de>
- Update to release 1.20
  * Addresses problems related to gcc 11 defaulting to DWARF5
    for -g.
  * Support SHN_XINDEX in st_shndx symbol indexes, to handle ELF
    objects with more than 65534 sections, for instance, which
    happens with kernels built with KCFLAGS="-ffunction-sections
  - fdata-sections", Other cases may include when using FG-ASLR,
    LTO.
  * Fix BTF variable generation for kernel modules, not skipping
    variables at offset zero.
  * Fix address size to match what is in the ELF file being
    processed, to fix using a 64-bit pahole binary to generate
    BTF for a 32-bit vmlinux image.
* Wed Jan  6 2021 Gary Ching-Pang Lin <glin@suse.com>
- Drop the version requirement of libdw-devel
  + DW_AT_alignment is defined in 1.19 so dwarves now is compatible
    with older libdw.
* Tue Nov 24 2020 Jan Engelhardt <jengelh@inai.de>
- Update to release 1.19
  * Support for split BTF.
  * Support cross-compiled ELF binaries with different endianness.
  * Support for showing typedefs for anonymous types, like structs,
    unions and enums.
* Tue Oct 27 2020 Jan Engelhardt <jengelh@inai.de>
- Add _service file / move to 1.18.g19 (863e6f0f)
  * btf_encoder: Ignore zero-sized ELF symbols
* Tue Oct 27 2020 Dirk Mueller <dmueller@suse.com>
- update to 1.18_15g040fd7f (bsc#1177921):
  * btf_encoder: Support cross-compiled ELF binaries with different endianness
  * strings: use BTF's string APIs for strings management
  * strings: Rename strings.h to avoid clashing with /usr/include/strings.h
  * dwarf_loader: increase the size of lookup hash map
  * btf_encoder: revamp how per-CPU variables are encoded
  * btf_encoder: Discard CUs after BTF encoding
  * btf_encoder: Fix emitting __ARRAY_SIZE_TYPE__ as index range type
  * btf_encoder: Use libbpf APIs to encode BTF type info
  * btf_loader: Use libbpf to load BTF
  * dwarves: Expose and maintain active debug info loader operations
  * btf_encoder: detect BTF encoding errors and exit
  * libbpf: Update to latest libbpf version
  * dwarf_loader: Conditionally define DW_AT_alignment
  * pahole: Implement --packed
  * man-pages: Fix 'coimbine' typo
- remove libbpf-Fix-libbpf-hashmap-on-I-LP32-architectures.patch (upstream)
* Fri Oct  2 2020 Jan Engelhardt <jengelh@inai.de>
- Update to release 1.18
  * pahole now should be segfault free when handling gdb test
    suit DWARF files, including ADA, FORTRAN, rust and dwp
    compressed files (the latter is refused).
  * Bail out on partial units for now, avoiding segfaults.
- Drop
  dwarves-dwarf_loader-Ignore-entries-in-a-DW_TAG_partial_unit-for-now.patch,
  dwarves-dwarf_loader-Bail-out-at-DW_TAG_imported_unit-tags.patch
  (merged)
- Add forced ELF symvers because of ABI breaks
* Mon Sep 21 2020 Tom de Vries <tdevries@suse.com>
- Backport fixes for segfault when processing dwz-ed kernel
  [bsc#1176095,https://github.com/acmel/dwarves/issues/10].
  * dwarves-dwarf_loader-Bail-out-at-DW_TAG_imported_unit-tags.patch
  * dwarves-dwarf_loader-Ignore-entries-in-a-DW_TAG_partial_unit-for-now.patch
* Wed Sep  2 2020 Jiri Slaby <jslaby@suse.com>
- add libbpf-Fix-libbpf-hashmap-on-I-LP32-architectures.patch
* Tue May 19 2020 Ismail Dönmez <idonmez@suse.com>
- Don't require libebl-plugins on suse_version >= 1550
* Sun Mar 15 2020 Jan Engelhardt <jengelh@inai.de>
- Update to release 1.17
  * Support raw BTF as available in /sys/kernel/btf/vmlinux.
  * Make --find_pointers_to consider unions.
  * Make --contains and --find_pointers_to honour --unions.
  * Add support for finding pointers to void.
  * Make --contains and --find_pointers_to to work with base types.
* Mon Dec 16 2019 Jan Engelhardt <jengelh@inai.de>
- Update to release 1.16
  * btf_loader: Add support for BTF_KIND_FUNC
* Thu Jun 27 2019 Jan Engelhardt <jengelh@inai.de>
- Update to new upstream release 1.15
  * Avoid null dereference with NULL configs
  * Fixup printing "const" early with "const void"
  * Fix recursively printing named structs in --expand_types
* Wed May  1 2019 dsterba@suse.cz
- version 1.13
  - Use of the recently introduced BTF deduplication algorithm
  - Introduce a 'btfdiff' utility
  - Add a BTF loader, i.e. 'pahole -F btf' allows pretty printing of structs
    and unions in the same fashion as with DWARF info
  - Better union support
  - Infer __attribute__((__packed__)) when structs have no alignment holes
  - Support DWARF5's DW_AT_alignment
  - Add a '--compile' option to 'pfunct'
* Thu Aug 16 2018 jengelh@inai.de
- Update to new upstream release 1.12
  * Added a BTF (BPF Type Format) encoder
  * Initial support for DW_TAG_partial_unit
  * Print cacheline boundaries in multiple union members
  * Show offsets at union members
* Mon Mar 26 2018 jengelh@inai.de
- dwarves needs elfutils to have the EBL plugins available
  to carry out its work. [boo#1049871]
- Update to new upstream release 1.11
  * dwarf_loader: Do not stop processing after finding
    unsupported tags
  * dwarf_loader: Support DW_FORM_data{4,8} for reading class
    member offsets
  * dwarf_fprintf: Handle DW_TAG_GNU_call_site{_parameter}
  * dwarf_loader: Add support for DW_TAG_restrict_type
  * dwarf_loader: Initial support for DW_TAG_unspecified_type
- Remove dwarves-no-DW_TAG_mutable.patch (already applied)
* Wed Feb  1 2017 mpluskal@suse.com
- Use cmake macros
* Thu Jan 22 2015 dimstar@opensuse.org
- Add dwarves-no-DW_TAG_mutable.patch: Fix build with newer
  libdwarf versions: DW_TAG_mutable_type was a mistake in an early
  DWARFv3 draft and was removed in the final version.
* Wed Oct 15 2014 jengelh@inai.de
- Drop libdwarf dependency; it is not needed for build (libdw
  is used instead)
* Wed Jun  6 2012 jengelh@inai.de
- Update to new upstream release 1.10
  * Initial DWARF4 support
  * Add stubs for some new GNU tags
  * Fix a crash when pahole is called with -R -S
* Sun Sep 18 2011 jengelh@medozas.de
- Remove redundant tags/sections from specfile
  (cf. packaging guidelines)
- Use %%_smp_mflags for parallel build
- Implement baselibs config
* Thu Aug 11 2011 jengelh@medozas.de
- Update to new upstream release 1.9
  * Add the "scncopy" tool - like object copy but tries not to change
  section content
  * pfunct: Introduce the -P option
  * dwarves fprintf: Add extra GNU tags
- Update group and description with something more detailed
- Remove redundant %%clean section
- Strip duplicate changelog
- Remove redundant manual requires on libraries
- Update URL to acme's new location
* Wed Apr 13 2011 mgorman@suse.de
- Remove %%debug_package directive which is breaking build
* Mon May  3 2010 lchiquitto@novell.com
- update to 1.8 release
* Fri Feb 29 2008 dmueller@suse.de
- update to 1.6 release
* Sat Dec  1 2007 dgollub@suse.de
- initial package of dwarves
